/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
// Methods for COptions class

#include <ctype.h>

#define NOPTS 8

// Global variables
static char optionsName[NOPTS][16]={
    "binMode",
    "binUserMin",
    "binUserMax",
    "nBins",
    "binUpdatePeriod",
    "axisMode",
    "axisXMin",
    "axisXMax",
};


#include "histTool.h"

COptions::COptions(void) :
    binMode(CHANNEL_RANGE),
    binUserMin(DEFAULT_BIN_MIN),
    binUserMax(DEFAULT_BIN_MAX),
    nBins(DEFAULT_NBINS),
    binUpdatePeriod(DEFAULT_UPDATE_PERIOD),
    axisMode(AUTO_SCALE_RANGE),
    axisXMin(binUserMin),
    axisXMax(binUserMax)
{
}

COptions::COptions(
  int binModeIn,
  double binUserMinIn,
  double binUserMaxIn,
  int nBinsIn,
  unsigned binUpdatePeriodIn,
  int axisModeIn,
  double axisXMinIn,
  double axisXMaxIn)
{
    binMode=binModeIn;
    binUserMin=binUserMinIn;
    binUserMax=binUserMaxIn;
    nBins=nBinsIn;
    binUpdatePeriod=binUpdatePeriodIn;
    axisMode=axisModeIn;
    axisXMin=axisXMinIn;
    axisXMax=axisXMaxIn;
}

COptions::~COptions(void)
{
}

int COptions::findInitFile(void)
{
    struct stat statinfo;

  // Look for init file in current directory with no dot, then in
  // $HOME, then in root with dot
    sprintf(initFilename,"%s",INITFILENAME);
    if(stat(initFilename,&statinfo) == -1) {
	char *home;
	
	home=getenv("HOME");
	
	sprintf(initFilename,"%s%s.%s",
	  home?home:"",DIR_DELIMITER_STRING,INITFILENAME);
	if(stat(initFilename,&statinfo) == -1) {
	    sprintf(initFilename,"%s.%s",DIR_DELIMITER_STRING,INITFILENAME);
	    if(stat(initFilename,&statinfo) == -1) {
		*initFilename='\0';
		return 0;
	    }
	}
    }
    return 1;
}

int COptions::read(void)
{
    FILE *fp;
    char line[80];
    char *name;
    char *value;
    char *status;
    int first=1;
    int i,j,opt=-1;

  // If there is no init filename, try to find it
    if(!*initFilename) {
	int fstatus;

	fstatus=findInitFile();
	if(!fstatus) return 0;
    }

  // Open the file and get the options
    fp=fopen(initFilename,"r");
    if(!fp) return 0;
    while(1) {
	status=fgets(line,80,fp);
	if(!status) {
	  // EOF or error, in either case we have done all we can
	    return 1;
	}
	if(first) {
	  // First line must be the program ID
	    if(strncmp(line,PROGID,strlen(PROGID))) {
		errMsg("Invalid initialization file: %s",initFilename);
		return 0;
	    }
	    first=0;
	    continue;
	}
	name=strtok(line,"=\n");
	if(!name) continue;
	value=strtok(NULL,"=\n");
	if(!value) continue;
	opt=NOPTS;
	for(i=0; i < NOPTS; i++) {
	  // Compare the strings by hand for platform independence
	    int len1=strlen(name);
	    int len2=strlen(optionsName[i]);
	    if(len1 != len2) continue;
	    int ok=1;
	    for(j=0; j < len1; j++) {
		if(toupper(name[j]) != toupper(optionsName[i][j])) {
		    ok=0;
		    break;
		}
	    }
	    if(ok) {
	      // Found it
		opt=i;
		break;
	    }
	}
	if(opt == NOPTS) break;
	switch(opt) {
	case 0:
	    binMode=atoi(value);
	case 1:
	    binUserMin=atof(value);
	    break;
	case 2:
	    binUserMax=atof(value);
	    break;
	case 3:
	    nBins=atoi(value);
	    break;
	case 4:
	    binUpdatePeriod=atol(value);
	    break;
	case 5:
	    axisMode=atoi(value);
	    break;
	case 6:
	    axisXMin=atof(value);
	    break;
	case 7:
	    axisXMax=atof(value);
	    break;
	}
    }
    fclose(fp);
    return 1;
}

int COptions::write(void)
{
    FILE *fp;
    int status;


  // Check the init filename
  // If there is no init filename, try to find it
    if(!*initFilename) {
	status=findInitFile();
	if(!status) {
	  // Use the HOME directory
	    char *home;
	    
	    home=getenv("HOME");
	    
	    sprintf(initFilename,"%s%s.%s",
	      home?home:"",DIR_DELIMITER_STRING,INITFILENAME);
	}
    }

  // Open the file
    fp=fopen(initFilename,"w");
    if(!fp) {
	errMsg("Error writing initialization file: %s",initFilename);
	*initFilename='\0';
	return 0;
    }
    
  // Write the options
    fprintf(fp,"%s\n",PROGID);
    fprintf(fp,"%s=%d\n",optionsName[0],binMode);
    fprintf(fp,"%s=%.15g\n",optionsName[1],binUserMin);
    fprintf(fp,"%s=%.15g\n",optionsName[2],binUserMax);
    fprintf(fp,"%s=%d\n",optionsName[3],nBins);
    fprintf(fp,"%s=%u\n",optionsName[4],binUpdatePeriod);
    fprintf(fp,"%s=%d\n",optionsName[5],axisMode);
    fprintf(fp,"%s=%.15g\n",optionsName[6],axisXMin);
    fprintf(fp,"%s=%.15g\n",optionsName[7],axisXMax);
    fclose(fp);

    return 1;
}
